/* -*- mode: c++; c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

/**
 * @author Régis Clouard - 2007-01-03
 */
    
/**
 * @file textmodel.cpp
 *
 * This widget displays a text or a collection contents.
 */

#include <QtGui>
#include <list>
#include <pandore.h>
using namespace pandore;

#include "frame.h"
#include "textmodel.h"

void TextModel::Init( const char *filename ) {
   setMaximumSize(1000,10000);
   storeFilename(filename);

   _contents = new QTextEdit();
   _contents->setReadOnly(true);
   QFont f = _contents->font();
   f.setPointSize(8);
   _contents->setFont(f); 
   _contents->setWordWrapMode(QTextOption::WrapAnywhere);

   QVBoxLayout *layout = new QVBoxLayout(this);
   layout->setContentsMargins(0,0,0,0);
   layout->addWidget(_contents);
}

TextModel::TextModel( const char *text, const char *filename ) {
   Init(filename);
   if (text != NULL) {
      _contents->insertPlainText(QString(text));
   }
}

/**
 * Comparison :  consider the second part of the name that are numbers.
 */
bool lowerFunction( const std::string& first, const std::string& second ) {
   unsigned pos1 =  first.find_first_of('.');
   unsigned pos2 =  second.find_first_of('.');

   // No numerical part in the name.
   if (pos1 == std::string::npos || pos2 == std::string::npos) {
      return first.compare(second) <= 0;
   }

   std::string first1 = first.substr(0,pos1);
   std::string second1 = second.substr(0,pos2);

   std::string first2 = first.substr(pos1);
   std::string second2 = second.substr(pos2);

   int order = first1.compare(second1);
   if (order == 0 ) {
      if (first2.length() < second2.length()) {
	 return true;
      } else if (first2.length() > second2.length()) {
	 return false;
      } else {
	 return first2.compare(second2) < 0;
      }
   } else {
      return order < 0;
   }
}

TextModel::TextModel( const Collection &cols, const char *filename ) {
   Init(filename);
   char tmp[512];
   const int MAXLENGTH = 500;
   std::list<std::string> l = cols.List();

   // Change the default string order....
   l.sort(lowerFunction);

   _contents->setTextColor(Qt::red);
   sprintf(tmp,"COLLECTION\nNumber of elements: %d\n", (int)l.size());
   _contents->insertPlainText(QString(tmp));

   std::list<std::string>::const_iterator i;
   for (i = l.begin(); i != l.end(); ++i) {
      BundledObject* bo = cols.Get(*i);
      char attributeType[255];
      char attributeName[255];

      strcpy(attributeType,bo->Type().c_str());
      strcpy(attributeName,i->c_str());
      _contents->setTextColor(Qt::blue);
      if (bo->NbrElements() < 2) {
	 sprintf(tmp, "  * %s (%s): %d item\n", attributeName, attributeType, bo->NbrElements());
      } else {
	 sprintf(tmp, "  * %s (%s): %d items\n", attributeName, attributeType, bo->NbrElements());
      }
      _contents->insertPlainText(QString(tmp));
      _contents->setTextColor(Qt::black);

      if (!strcasecmp(attributeType,"Double")) {
         Double &f = cols.GETVALUE(attributeName, Double);
         sprintf(tmp, "%f\n", f);
         _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Float")) {
	 Float &f = cols.GETVALUE(attributeName, Float);
	 sprintf(tmp, "%f\n", f);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Long")) {
	 Long &l = cols.GETVALUE(attributeName, Long);
	 sprintf(tmp, "%d\n", l);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Ulong")) {
	 Ulong &l = cols.GETVALUE(attributeName, Ulong);
	 sprintf(tmp, "%u\n", l);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Short")) {
	 Short &l = cols.GETVALUE(attributeName, Short);
	 sprintf(tmp, "%hd\n", l);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Ushort")) {
	 Ushort &l = cols.GETVALUE(attributeName, Ushort);
	 sprintf(tmp, "%hu\n", l);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Char")) {
	 Char &uc = cols.GETVALUE(attributeName, Char);
	 sprintf(tmp, "%d\n", (int)uc);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Uchar")) {
	 Uchar &uc = cols.GETVALUE(attributeName, Uchar);
	 sprintf(tmp, "%d\n",  (int)uc);
	 _contents->insertPlainText(QString(tmp));
      } else if (!strcasecmp(attributeType,"Array:Double")) {
	 Double *tab = cols.GETARRAY(attributeName, Double);
	 Long taille = cols.GETARRAYSIZE(attributeName, Double);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
            sprintf(tmp, "%f\t",  tab[j]);
            _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      } else if (!strcasecmp(attributeType, "Array:Float")) {
	 Float *tab = cols.GETARRAY(attributeName, Float);
	 Long taille = cols.GETARRAYSIZE(attributeName, Float);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%f\t",  tab[j]);
	    _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      } else if (!strcasecmp(attributeType, "Array:Long")) {
	 Long *tab = cols.GETARRAY(attributeName, Long);
	 Long taille = cols.GETARRAYSIZE(attributeName, Long);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%d\t", tab[j]);
	    _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      } else if (!strcasecmp(attributeType, "Array:Ulong")) {
	 Ulong *tab = cols.GETARRAY(attributeName, Ulong);
	 pandore::Long taille = cols.GETARRAYSIZE(attributeName, Ulong);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%u\t",  tab[j]);
	    _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      } else if (!strcasecmp(attributeType, "Array:Short")) {
	 Short *tab = cols.GETARRAY(attributeName, Short);
	 Long taille = cols.GETARRAYSIZE(attributeName, Short);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%hd\t", tab[j]);
	    _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      }  else if (!strcasecmp(attributeType, "Array:UShort")) {
         Ushort *tab = cols.GETARRAY(attributeName, Ushort);
         Long taille = cols.GETARRAYSIZE(attributeName, Ushort);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%hu\t", tab[j]);
	    _contents->insertPlainText(QString(tmp));
         }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
         _contents->insertPlainText(QString("\n"));
      } else if (!strcasecmp(attributeType, "Array:Char")) {
	 Char *tab = cols.GETARRAY(attributeName, Char);
	 Long taille = cols.GETARRAYSIZE(attributeName, Char);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%d\t", (int)tab[j]);
	    _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      } else if (!strcasecmp(attributeType, "Array:Uchar")) {
	 Uchar *tab = cols.GETARRAY(attributeName, Uchar);
	 Long taille = cols.GETARRAYSIZE(attributeName, Uchar);
	 for (int j = 0; j < MIN(taille, MAXLENGTH); j++) {
	    sprintf(tmp, "%d\t", (int)tab[j]);
	    _contents->insertPlainText(QString(tmp));
	 }
	 if (taille>MAXLENGTH) {
	    _contents->insertPlainText(QString(" (... file too large)"));
	 }
	 _contents->insertPlainText(QString("\n"));
      } else {
	 _contents->insertPlainText(QString(" unprintable value(s)\n"));
      }
   }
   _contents->moveCursor(QTextCursor::Start);
}

void TextModel::keyPressEvent( QKeyEvent *e ) {
   switch (e->key()) {
   case Qt::Key_O :
      _frame->openImage();
      break;
   case Qt::Key_Q :
      _frame->closeApplication();
   default :
      break;
   }
}

void TextModel::setParent( Frame *frame ) {
   _frame = frame;
   frame->setContent(this);
}
